package com.qmth.teachcloud.common.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicUserCourse;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.mapper.BasicUserCourseMapper;
import com.qmth.teachcloud.common.service.BasicUserCourseService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 用户和课程关联表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class BasicUserCourseServiceImpl extends ServiceImpl<BasicUserCourseMapper, BasicUserCourse> implements BasicUserCourseService {

    @Override
    public void removeByUserId(Long userId) {
        UpdateWrapper<BasicUserCourse> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(BasicUserCourse::getUserId, userId);
        this.baseMapper.delete(updateWrapper);
    }

    @Override
    public void saveBatch(SysUser sysUser) {
        Long[] courseIds = sysUser.getCourseIds();
        List<BasicUserCourse> list = new ArrayList<>();
        for (Long courseId : courseIds) {
            BasicUserCourse userCourse = new BasicUserCourse();
            userCourse.setUserId(sysUser.getId());
            userCourse.setCourseId(courseId);
            userCourse.setCreateId(sysUser.getId());
            userCourse.setCreateTime(System.currentTimeMillis());
            list.add(userCourse);
        }
        this.saveBatch(list);
    }

    @Transactional
    @Override
    public void updateByCourseIdAndClazzList(Long courseId, List<String> clazzList, SysUser sysUser) {
        List<BasicUserCourse> basicUserCourseList = this.list(new QueryWrapper<BasicUserCourse>().lambda()
                .eq(BasicUserCourse::getCourseId, courseId));

        // 寻找要删除的目标（1.班级不为空 2.老班级不被新班级集合包含）
        List<BasicUserCourse> willUpdateList = new ArrayList<>();
        List<Long> willDeleteIdList = new ArrayList<>();
        List<String> willAddClazz = new ArrayList<>(clazzList);
        List<BasicUserCourse> willAddList = new ArrayList<>();

        for (BasicUserCourse old : basicUserCourseList) {
            String clazz = old.getClazz();
            Long id = old.getId();
            if (SystemConstant.strNotNull(clazz)){
                if (clazzList.contains(clazz)){
                    // 要更新时间的旧关系
                    old.setOrgId(sysUser.getOrgId());
                    old.setUpdateId(sysUser.getUpdateId());
                    willUpdateList.add(old);
                    willAddClazz.remove(clazz);
                }else {
                    // 要被删除的旧关系
                    willDeleteIdList.add(id);
                }
            }
        }

        // 删掉全部
        if (willDeleteIdList.size() > 0) {
            this.removeByIds(willDeleteIdList);
        }

        // 更新
        this.updateBatchById(willUpdateList);

        // 新增
        for (String addClazz : willAddClazz) {
            BasicUserCourse basicUserCourse = new BasicUserCourse();
            basicUserCourse.setId(SystemConstant.getDbUuid());
            basicUserCourse.setSchoolId(sysUser.getSchoolId());
            basicUserCourse.setOrgId(sysUser.getOrgId());
            basicUserCourse.setCourseId(courseId);
            basicUserCourse.setClazz(addClazz);
            basicUserCourse.setCreateId(sysUser.getId());
            willAddList.add(basicUserCourse);
        }
        this.saveBatch(willAddList);
    }

    @Override
    public List<BasicUserCourse> listBySchoolIdAndCourseCode(Long schoolId, String courseCode) {
        return this.baseMapper.listBySchoolIdAndCourseCode(schoolId, courseCode);
    }
}
